import tkinter as tk
from tkinter import colorchooser, filedialog, messagebox
from tkinterdnd2 import TkinterDnD, DND_FILES
import numpy as np
from PIL import Image, PngImagePlugin
import json
import os

SETTINGS_FILE = "settings.json"

def generate_noise_image(width, height, noise_intensity, base_color):
    base_r, base_g, base_b = base_color
    noise = np.random.randint(0, noise_intensity, (height, width, 3), dtype=np.uint8)
    noise_image_data = np.clip(noise + [base_r, base_g, base_b], 0, 255).astype(np.uint8)
    image = Image.fromarray(noise_image_data, mode='RGB')
    
    # メタデータに設定値を書き込む
    metadata = PngImagePlugin.PngInfo()
    metadata.add_text("settings", json.dumps({
        "width": width,
        "height": height,
        "noise_intensity": noise_intensity,
        "base_color": base_color,
        "mode": "normal",
        "gradient_direction": "horizontal",
        "mix_point": 50
    }))
    return image, metadata

def generate_gradient_noise_image(width, height, noise_intensity, base_color, direction):
    base_r, base_g, base_b = base_color
    noise = np.random.randint(0, noise_intensity, (height, width, 3), dtype=np.uint8)
    if direction == "vertical":
        gradient = np.linspace(1, 0, height).reshape(height, 1, 1)
    else:  # horizontal
        gradient = np.linspace(1, 0, width).reshape(1, width, 1)
    gradient = gradient * np.array([base_r, base_g, base_b])
    noise_image_data = np.clip(noise + gradient, 0, 255).astype(np.uint8)
    image = Image.fromarray(noise_image_data, mode='RGB')
    
    # メタデータに設定値を書き込む
    metadata = PngImagePlugin.PngInfo()
    metadata.add_text("settings", json.dumps({
        "width": width,
        "height": height,
        "noise_intensity": noise_intensity,
        "base_color": base_color,
        "mode": "gradient",
        "gradient_direction": direction,
        "mix_point": 50
    }))
    return image, metadata

def generate_gradient_2color_noise_image(width, height, noise_intensity, base_colors, direction):
    noise = np.random.randint(0, noise_intensity, (height, width, 3), dtype=np.uint8)
    color1, color2 = np.array(base_colors[0]), np.array(base_colors[1])

    if direction == "vertical":
        gradient = np.zeros((height, width, 3), dtype=np.float32)
        for i in range(height):
            alpha = i / height
            gradient[i, :, :] = (1 - alpha) * color1 + alpha * color2
    else:  # horizontal
        gradient = np.zeros((height, width, 3), dtype=np.float32)
        for i in range(width):
            alpha = i / width
            gradient[:, i, :] = (1 - alpha) * color1 + alpha * color2

    noise_image_data = np.clip(noise + gradient, 0, 255).astype(np.uint8)
    image = Image.fromarray(noise_image_data, mode='RGB')
    
    # メタデータに設定値を書き込む
    metadata = PngImagePlugin.PngInfo()
    metadata.add_text("settings", json.dumps({
        "width": width,
        "height": height,
        "noise_intensity": noise_intensity,
        "base_colors": base_colors,
        "mode": "gradient_2color",
        "gradient_direction": direction
    }))
    return image, metadata

def generate_gradient_2color_with_mix_point_noise_image(width, height, noise_intensity, base_colors, direction, mix_point):
    noise = np.random.randint(0, noise_intensity, (height, width, 3), dtype=np.uint8)
    color1, color2 = np.array(base_colors[0]), np.array(base_colors[1])
    mix_point = float(mix_point) / 100

    if direction == "vertical":
        gradient = np.zeros((height, width, 3), dtype=np.float32)
        for i in range(height):
            alpha = (i - mix_point * height) / (height * (1 - mix_point))
            alpha = np.clip(alpha, 0, 1)
            gradient[i, :, :] = (1 - alpha) * color1 + alpha * color2
    else:  # horizontal
        gradient = np.zeros((height, width, 3), dtype=np.float32)
        for i in range(width):
            alpha = (i - mix_point * width) / (width * (1 - mix_point))
            alpha = np.clip(alpha, 0, 1)
            gradient[:, i, :] = (1 - alpha) * color1 + alpha * color2

    noise_image_data = np.clip(noise + gradient, 0, 255).astype(np.uint8)
    image = Image.fromarray(noise_image_data, mode='RGB')
    
    # メタデータに設定値を書き込む
    metadata = PngImagePlugin.PngInfo()
    metadata.add_text("settings", json.dumps({
        "width": width,
        "height": height,
        "noise_intensity": noise_intensity,
        "base_colors": base_colors,
        "mode": "gradient_2color_with_mix_point",
        "gradient_direction": direction,
        "mix_point": mix_point * 100
    }))
    return image, metadata

def choose_color(index):
    initial_color = tuple(map(int, base_colors[index].get().strip('()').split(',')))
    color_code = colorchooser.askcolor(color=initial_color, title="Choose base color")
    if color_code[0]:
        base_colors[index].set(str(tuple(map(int, color_code[0]))))
    return color_code[0] if color_code[0] else initial_color

def save_file_dialog():
    file_path = filedialog.asksaveasfilename(defaultextension=".png", filetypes=[("PNG files", "*.png"), ("All files", "*.*")])
    return file_path

def generate_image():
    try:
        width = int(entry_width.get())
        height = int(entry_height.get())
        noise_intensity = int(entry_noise_intensity.get())
        selected_mode = mode.get()
        selected_direction = gradient_direction.get()
        mix_point = int(entry_mix_point.get())

        if selected_mode == "gradient_2color_with_mix_point":
            base_color_values = [tuple(map(int, base_colors[i].get().strip('()').split(','))) for i in range(len(base_colors))]
            noise_image, metadata = generate_gradient_2color_with_mix_point_noise_image(width, height, noise_intensity, base_color_values, selected_direction, mix_point)
        elif selected_mode == "gradient_2color":
            base_color_values = [tuple(map(int, base_colors[i].get().strip('()').split(','))) for i in range(len(base_colors))]
            noise_image, metadata = generate_gradient_2color_noise_image(width, height, noise_intensity, base_color_values, selected_direction)
        elif selected_mode == "gradient":
            base_color_value = tuple(map(int, base_colors[0].get().strip('()').split(',')))
            noise_image, metadata = generate_gradient_noise_image(width, height, noise_intensity, base_color_value, selected_direction)
        else:
            base_color_value = tuple(map(int, base_colors[0].get().strip('()').split(',')))
            noise_image, metadata = generate_noise_image(width, height, noise_intensity, base_color_value)
        
        file_path = save_file_dialog()

        if file_path:
            noise_image.save(file_path, pnginfo=metadata)
            messagebox.showinfo("Success", "Image saved successfully!")
            save_settings(width, height, noise_intensity, base_colors, selected_mode, selected_direction, mix_point)
    except ValueError:
        messagebox.showerror("Error", "Please enter valid numeric values for width, height, noise intensity, and mix point.")
    except Exception as e:
        messagebox.showerror("Error", f"An unexpected error occurred: {e}")

def load_settings():
    if os.path.exists(SETTINGS_FILE):
        try:
            with open(SETTINGS_FILE, 'r') as f:
                return json.load(f)
        except json.JSONDecodeError:
            messagebox.showwarning("Warning", "Settings file is corrupted. Using default settings.")
            return {"width": 512, "height": 512, "noise_intensity": 50, "base_colors": [(0, 0, 0), (255, 255, 255)], "mode": "normal", "gradient_direction": "horizontal", "mix_point": 50}
    return {"width": 512, "height": 512, "noise_intensity": 50, "base_colors": [(0, 0, 0), (255, 255, 255)], "mode": "normal", "gradient_direction": "horizontal", "mix_point": 50}

def save_settings(width, height, noise_intensity, base_colors, mode, direction, mix_point):
    settings = {
        "width": width,
        "height": height,
        "noise_intensity": noise_intensity,
        "base_colors": [tuple(map(int, color.get().strip('()').split(','))) for color in base_colors],
        "mode": mode,
        "gradient_direction": direction,
        "mix_point": mix_point
    }
    try:
        with open(SETTINGS_FILE, 'w') as f:
            json.dump(settings, f)
    except IOError as e:
        messagebox.showerror("Error", f"Failed to save settings: {e}")

def on_drop(event):
    file_path = event.data.strip('{}')
    if file_path.lower().endswith(".png"):
        try:
            with Image.open(file_path) as img:
                metadata = img.info.get("settings")
                if metadata:
                    settings = json.loads(metadata)
                    entry_width.delete(0, tk.END)
                    entry_width.insert(0, settings["width"])
                    entry_height.delete(0, tk.END)
                    entry_height.insert(0, settings["height"])
                    entry_noise_intensity.delete(0, tk.END)
                    entry_noise_intensity.insert(0, settings["noise_intensity"])
                    mode.set(settings.get("mode", "normal"))
                    gradient_direction.set(settings.get("gradient_direction", "horizontal"))
                    entry_mix_point.delete(0, tk.END)
                    entry_mix_point.insert(0, settings["mix_point"])
                    for i in range(len(base_colors)):
                        base_colors[i].set(str(tuple(settings["base_colors"][i])))
                    update_color_fields()
                    messagebox.showinfo("Success", "Settings loaded from image metadata.")
                else:
                    messagebox.showwarning("Warning", "No settings metadata found in the image.")
        except Exception as e:
            messagebox.showerror("Error", f"Failed to load settings from image: {e}")
    else:
        messagebox.showwarning("Warning", "Please drop a PNG file.")

def update_color_fields():
    selected_mode = mode.get()
    if selected_mode == "gradient_2color_with_mix_point":
        entry_base_color1.grid()
        color_button1.grid()
        entry_base_color2.grid()
        color_button2.grid()
        gradient_direction_label.grid()
        horizontal_rb.grid()
        vertical_rb.grid()
        mix_point_label.grid()
        entry_mix_point.grid()
    elif selected_mode == "gradient_2color":
        entry_base_color1.grid()
        color_button1.grid()
        entry_base_color2.grid()
        color_button2.grid()
        gradient_direction_label.grid()
        horizontal_rb.grid()
        vertical_rb.grid()
        mix_point_label.grid_remove()
        entry_mix_point.grid_remove()
    elif selected_mode == "gradient":
        entry_base_color1.grid()
        color_button1.grid()
        entry_base_color2.grid_remove()
        color_button2.grid_remove()
        gradient_direction_label.grid()
        horizontal_rb.grid()
        vertical_rb.grid()
        mix_point_label.grid_remove()
        entry_mix_point.grid_remove()
    else:
        entry_base_color1.grid()
        color_button1.grid()
        entry_base_color2.grid_remove()
        color_button2.grid_remove()
        gradient_direction_label.grid_remove()
        horizontal_rb.grid_remove()
        vertical_rb.grid_remove()
        mix_point_label.grid_remove()
        entry_mix_point.grid_remove()

# GUIの作成
root = TkinterDnD.Tk()
root.title("Noise Image Generator")

# 前回の設定を読み込む
settings = load_settings()

# 画像サイズ入力
tk.Label(root, text="Width:").grid(row=0, column=0)
entry_width = tk.Entry(root)
entry_width.grid(row=0, column=1)
entry_width.insert(0, settings["width"])

tk.Label(root, text="Height:").grid(row=1, column=0)
entry_height = tk.Entry(root)
entry_height.grid(row=1, column=1)
entry_height.insert(0, settings["height"])

# ノイズ強度入力
tk.Label(root, text="Noise Intensity (0-255):").grid(row=2, column=0)
entry_noise_intensity = tk.Entry(root)
entry_noise_intensity.grid(row=2, column=1)
entry_noise_intensity.insert(0, settings["noise_intensity"])

# モード選択
mode = tk.StringVar()
mode.set(settings["mode"])
tk.Label(root, text="Mode:").grid(row=3, column=0)
tk.Radiobutton(root, text="Normal", variable=mode, value="normal", command=update_color_fields).grid(row=3, column=1)
tk.Radiobutton(root, text="Gradient (1 Color)", variable=mode, value="gradient", command=update_color_fields).grid(row=3, column=2)
tk.Radiobutton(root, text="Gradient (2 Colors)", variable=mode, value="gradient_2color", command=update_color_fields).grid(row=3, column=3)
tk.Radiobutton(root, text="Gradient (2 Colors with Mix Point)", variable=mode, value="gradient_2color_with_mix_point", command=update_color_fields).grid(row=3, column=4)

# グラデーション方向選択
gradient_direction = tk.StringVar()
gradient_direction.set(settings["gradient_direction"])
gradient_direction_label = tk.Label(root, text="Gradient Direction:")
gradient_direction_label.grid(row=4, column=0)
horizontal_rb = tk.Radiobutton(root, text="Horizontal", variable=gradient_direction, value="horizontal")
horizontal_rb.grid(row=4, column=1)
vertical_rb = tk.Radiobutton(root, text="Vertical", variable=gradient_direction, value="vertical")
vertical_rb.grid(row=4, column=2)

# ベースカラー入力
base_colors = [tk.StringVar(), tk.StringVar()]
tk.Label(root, text="Base Color 1 (R, G, B):").grid(row=5, column=0)
entry_base_color1 = tk.Entry(root, textvariable=base_colors[0])
entry_base_color1.grid(row=5, column=1)
base_colors[0].set(str(tuple(settings["base_colors"][0])))

color_button1 = tk.Button(root, text="Choose Color 1", command=lambda: choose_color(0))
color_button1.grid(row=5, column=2)

tk.Label(root, text="Base Color 2 (R, G, B):").grid(row=6, column=0)
entry_base_color2 = tk.Entry(root, textvariable=base_colors[1])
entry_base_color2.grid(row=6, column=1)
base_colors[1].set(str(tuple(settings["base_colors"][1])))

color_button2 = tk.Button(root, text="Choose Color 2", command=lambda: choose_color(1))
color_button2.grid(row=6, column=2)

# 交わる割合入力
mix_point_label = tk.Label(root, text="Mixing Point (%):")
mix_point_label.grid(row=7, column=0)
entry_mix_point = tk.Entry(root)
entry_mix_point.grid(row=7, column=1)
entry_mix_point.insert(0, settings["mix_point"])

# ベースカラー2をモードによって表示/非表示
update_color_fields()

# 生成ボタン
generate_button = tk.Button(root, text="Generate Image", command=generate_image)
generate_button.grid(row=8, columnspan=3)

# ドラッグ＆ドロップイベントのバインド
root.drop_target_register(DND_FILES)
root.dnd_bind('<<Drop>>', on_drop)

root.mainloop()
